@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the Qt Virtual Keyboard module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtVirtualKeyboard/qvirtualkeyboardinputengine.h>
 #include <QtVirtualKeyboard/qvirtualkeyboardinputcontext.h>
@@ -35,6 +9,8 @@
 #include <QtVirtualKeyboard/qvirtualkeyboardtrace.h>
 #include <QtVirtualKeyboard/private/virtualkeyboarddebug_p.h>
 
+#include <QQmlContext>
+#include <QQmlEngine>
 #include <QTimerEvent>
 #include <QtCore/private/qobject_p.h>
 
@@ -76,7 +52,7 @@ public:
                 accept = fallbackInputMethod->keyEvent(key, text, modifiers);
             }
             emit q->virtualKeyClicked(key, text, modifiers, isAutoRepeat);
-        } else if (QT_VIRTUALKEYBOARD_FORCE_EVENTS_WITHOUT_FOCUS) {
+        } else if (QtVirtualKeyboard::forceEventsWithoutFocus()) {
             accept = fallbackInputMethod->keyEvent(key, text, modifiers);
             emit q->virtualKeyClicked(key, text, modifiers, isAutoRepeat);
         } else {
@@ -124,8 +100,8 @@ private:
 /*!
     \qmltype InputEngine
     \inqmlmodule QtQuick.VirtualKeyboard
-    \ingroup qtvirtualkeyboard-qml
-    \instantiates QVirtualKeyboardInputEngine
+    \ingroup qtvirtualkeyboard-internal-qml
+    \nativetype QVirtualKeyboardInputEngine
     \brief Maps the user input to the input methods.
 
     The input engine is responsible for routing input events to input
@@ -139,6 +115,7 @@ private:
 /*!
     \class QVirtualKeyboardInputEngine
     \inmodule QtVirtualKeyboard
+    \ingroup qtvirtualkeyboard-cpp-for-devs
     \brief The InputEngine class provides an input engine
     that supports C++ and QML integration.
 
@@ -476,7 +453,7 @@ QList<int> QVirtualKeyboardInputEngine::patternRecognitionModes() const
     if (patterRecognitionModeList.isEmpty())
         return resultList;
     resultList.reserve(patterRecognitionModeList.size());
-    for (const PatternRecognitionMode &patternRecognitionMode : qAsConst(patterRecognitionModeList))
+    for (const PatternRecognitionMode &patternRecognitionMode : std::as_const(patterRecognitionModeList))
         resultList.append(static_cast<int>(patternRecognitionMode));
     return resultList;
 }
@@ -539,8 +516,14 @@ QVirtualKeyboardTrace *QVirtualKeyboardInputEngine::traceBegin(
     if (!d->inputMethod->patternRecognitionModes().contains(patternRecognitionMode))
         return nullptr;
     QVirtualKeyboardTrace *trace = d->inputMethod->traceBegin(traceId, patternRecognitionMode, traceCaptureDeviceInfo, traceScreenInfo);
-    if (trace)
+    if (trace) {
+        if (QQmlContext *context = QQmlEngine::contextForObject(this)) {
+            if (QQmlEngine *engine = context->engine()) {
+                engine->setObjectOwnership(trace, QQmlEngine::CppOwnership);
+            }
+        }
         trace->setTraceId(traceId);
+    }
     return trace;
 }
 
@@ -677,7 +660,7 @@ void QVirtualKeyboardInputEngine::updateSelectionListModels()
     }
 
     // Deallocate inactive selection lists
-    for (const QVirtualKeyboardSelectionListModel::Type &selectionListType : qAsConst(inactiveSelectionLists)) {
+    for (const QVirtualKeyboardSelectionListModel::Type &selectionListType : std::as_const(inactiveSelectionLists)) {
         const auto it = d->selectionListModels.constFind(selectionListType);
         if (it != d->selectionListModels.cend()) {
             it.value()->setDataSource(nullptr, selectionListType);
